@@ -19,6 +19,62 @@ null = None
 true = True
 false = False
 
+today = date.today().strftime("%b-%d-%Y")
+
+MongoDB = ControllerMongoDB()
+PGDB = ControllerPostgresDB()
+RedShift = ControllerRedShift()
+
+MongoDB.get_con_collect(collection_name='fact')
+
+
+def get_info_by_transaction_uid(transaction_uid):
+    MongoDB.get_con_collect(collection_name='analytic')
+    
+    responde_transaction = MongoDB.con_db.find_one({'_id': transaction_uid})
+
+    if not responde_transaction:
+        responde_transaction = MongoDB.con_db.find_one({"history-track.USER.transaction_uid": transaction_uid})
+
+    
+    if responde_transaction is None :
+        return 
+
+    reponse = dict()
+    reponse['account_uid'] =responde_transaction['history-track'][0]['USER']['account_uid']
+    reponse['amount'] =responde_transaction['history-track'][0]['USER']['raw']['amount']
+    reponse['currency'] =responde_transaction['history-track'][0]['USER']['raw']['currency']
+    reponse['bank_name'] =responde_transaction['history-track'][0]['USER']['raw']['account']['institution']['name']
+    reponse['account_type'] =responde_transaction['history-track'][0]['USER']['raw']['account']['type']
+    reponse['bank_type'] =responde_transaction['history-track'][0]['USER']['raw']['account']['institution']['type']
+    if 'message_type' in responde_transaction['history-track'][0]['CASE']:
+        reponse['message_type'] =responde_transaction['history-track'][0]['CASE']['message_type']
+
+
+    print(reponse)
+
+    
+
+    # ['raw']['account']['name']
+    # ['raw']['account']['number']
+    # ['raw']['account']['id']
+    # ['raw']['account']['category']
+    
+    reponse['collected_at'] =responde_transaction['history-track'][0]['USER']['raw']['collected_at']
+    reponse['original_created_date'] =responde_transaction['history-track'][0]['USER']['raw']['value_date']
+    reponse['flow_type'] =responde_transaction['history-track'][0]['USER']['raw']['type']
+
+    
+    # source description
+    reponse['source_description'] =responde_transaction['history-track'][0]['USER']['description']
+    reponse['cat_uid_def_usr'] =responde_transaction['history-track'][0]['CASE']['categorization']['category_uid']
+    reponse['subcat_uid_def_usr'] =responde_transaction['history-track'][0]['CASE']['categorization']['subcategory_uid']
+    reponse['nickname_algo'] =responde_transaction['history-track'][0]['CASE']['categorization']['description']
+    reponse['categorized'] =responde_transaction['history-track'][0]['CASE']['categorized']
+    reponse['model_classification_version'] =responde_transaction['history-track'][0]['MODEL']['model_uid']
+    
+    return reponse
+
 class Summary:
     """
     """
@@ -76,6 +132,7 @@ class Summary:
                     "message_type_history":"$history-track.message_type",
                     "message_type_old":"$history-track.type" ,
 
+ 
                     "amount" :"$amount",
                     "model_nlp_version":"$history-track.model_nlp_version", # pre
                     "model_classification_version":"$history-track.model_classification_version",
@@ -183,10 +240,8 @@ class Summary:
         ]
        
         return pd.json_normalize(pd.DataFrame(list( self.MongoDB.con_db.aggregate(self.query_summary_category) ))['_id']) 
+    
 
-MongoDB = ControllerMongoDB()
-PGDB = ControllerPostgresDB()
-RedShift = ControllerRedShift()
 
 RedShift.get_conection()
 
@@ -196,59 +251,60 @@ front_date = today.strftime("%Y-%m-%d")
 print(today)
 print("front_date: " , front_date)
 
-s = Summary(front_date,num_days = 240)
+s = Summary(front_date,num_days = 75)
 r = s.agg_master()
 
 
-r['business_uid_def_usr'] = r['business_uid_def_usr'].apply(lambda x: x) + r['business_uid_def_usr_'].apply(lambda x: x) + r['business_uid_def_usr_info'].apply(lambda x: x)
+def refill():
+    r['business_uid_def_usr'] = r['business_uid_def_usr'].apply(lambda x: x) + r['business_uid_def_usr_'].apply(lambda x: x) + r['business_uid_def_usr_info'].apply(lambda x: x)
 
-del r['business_uid_def_usr_']
-del r['business_uid_def_usr_info']
+    del r['business_uid_def_usr_']
+    del r['business_uid_def_usr_info']
 
 
-r['asset_uid_def_usr'] = r['asset_uid_def_usr'].apply(lambda x: x) + r['asset_uid_def_usr_'].apply(lambda x: x) + r['asset_uid_def_usr_info'].apply(lambda x: x)
+    r['asset_uid_def_usr'] = r['asset_uid_def_usr'].apply(lambda x: x) + r['asset_uid_def_usr_'].apply(lambda x: x) + r['asset_uid_def_usr_info'].apply(lambda x: x)
 
-del r['asset_uid_def_usr_info']
-del r['asset_uid_def_usr_']
+    del r['asset_uid_def_usr_info']
+    del r['asset_uid_def_usr_']
 
-r['frequency_uid_def_usr'] = r['frequency_uid_def_usr'].apply(lambda x: x) + r['frequency_uid_def_usr_'].apply(lambda x: x) + r['frequency_uid_def_usr_info'].apply(lambda x: x)
+    r['frequency_uid_def_usr'] = r['frequency_uid_def_usr'].apply(lambda x: x) + r['frequency_uid_def_usr_'].apply(lambda x: x) + r['frequency_uid_def_usr_info'].apply(lambda x: x)
 
-del r['frequency_uid_def_usr_info']
-del r['frequency_uid_def_usr_']
+    del r['frequency_uid_def_usr_info']
+    del r['frequency_uid_def_usr_']
 
-r['income_source_uid_def_usr'] = r['income_source_uid_def_usr'].apply(lambda x: x) + r['income_source_uid_def_usr_'].apply(lambda x: x) + r['income_source_uid_def_usr_info'].apply(lambda x: x)
+    r['income_source_uid_def_usr'] = r['income_source_uid_def_usr'].apply(lambda x: x) + r['income_source_uid_def_usr_'].apply(lambda x: x) + r['income_source_uid_def_usr_info'].apply(lambda x: x)
 
-del r['income_source_uid_def_usr_info']
-del r['income_source_uid_def_usr_']
+    del r['income_source_uid_def_usr_info']
+    del r['income_source_uid_def_usr_']
 
 
-r['is_loan_received_def_usr'] = r['is_loan_received_def_usr'].apply(lambda x: x) + r['is_loan_received_def_usr_'].apply(lambda x: x) + r['is_loan_received_def_usr_info'].apply(lambda x: x)
+    r['is_loan_received_def_usr'] = r['is_loan_received_def_usr'].apply(lambda x: x) + r['is_loan_received_def_usr_'].apply(lambda x: x) + r['is_loan_received_def_usr_info'].apply(lambda x: x)
 
-del r['is_loan_received_def_usr_info']
-del r['is_loan_received_def_usr_']
+    del r['is_loan_received_def_usr_info']
+    del r['is_loan_received_def_usr_']
 
-r['is_loan_requested_def_usr'] = r['is_loan_requested_def_usr'].apply(lambda x: x) + r['is_loan_requested_def_usr_'].apply(lambda x: x) + r['is_loan_requested_def_usr_info'].apply(lambda x: x)
+    r['is_loan_requested_def_usr'] = r['is_loan_requested_def_usr'].apply(lambda x: x) + r['is_loan_requested_def_usr_'].apply(lambda x: x) + r['is_loan_requested_def_usr_info'].apply(lambda x: x)
 
-del r['is_loan_requested_def_usr_info']
-del r['is_loan_requested_def_usr_']
+    del r['is_loan_requested_def_usr_info']
+    del r['is_loan_requested_def_usr_']
 
 
-r['is_membership_def_usr'] = r['is_membership_def_usr'].apply(lambda x: x) + r['is_membership_def_usr_'].apply(lambda x: x) + r['is_membership_def_usr_info'].apply(lambda x: x)
+    r['is_membership_def_usr'] = r['is_membership_def_usr'].apply(lambda x: x) + r['is_membership_def_usr_'].apply(lambda x: x) + r['is_membership_def_usr_info'].apply(lambda x: x)
 
-del r['is_membership_def_usr_info']
-del r['is_membership_def_usr_']
+    del r['is_membership_def_usr_info']
+    del r['is_membership_def_usr_']
 
 
-r['person_uid_def_usr'] = r['person_uid_def_usr'].apply(lambda x: x) + r['person_uid_def_usr_'].apply(lambda x: x) + r['person_uid_def_usr_info'].apply(lambda x: x)
+    r['person_uid_def_usr'] = r['person_uid_def_usr'].apply(lambda x: x) + r['person_uid_def_usr_'].apply(lambda x: x) + r['person_uid_def_usr_info'].apply(lambda x: x)
 
-del r['person_uid_def_usr_info']
-del r['person_uid_def_usr_']
+    del r['person_uid_def_usr_info']
+    del r['person_uid_def_usr_']
 
 
-r['special_moment_uid_def_usr'] = r['special_moment_uid_def_usr'].apply(lambda x: x) + r['special_moment_uid_def_usr_'].apply(lambda x: x) + r['special_moment_uid_def_usr_info'].apply(lambda x: x)
+    r['special_moment_uid_def_usr'] = r['special_moment_uid_def_usr'].apply(lambda x: x) + r['special_moment_uid_def_usr_'].apply(lambda x: x) + r['special_moment_uid_def_usr_info'].apply(lambda x: x)
 
-del r['special_moment_uid_def_usr_info']
-del r['special_moment_uid_def_usr_']
+    del r['special_moment_uid_def_usr_info']
+    del r['special_moment_uid_def_usr_']
 
 
 for col in sorted(r.columns.tolist()):
@@ -263,10 +319,10 @@ r.replace(np.nan, None, inplace=True)
 r.replace(pd.NaT, None, inplace=True)
 tmp = {'created_at_homologation': pd.NaT}
 
-print(r.columns.tolist())
+
 
 for idx ,item in enumerate(r.to_dict(orient='records')):
-    print(idx, ": ",item['transaction_uid'])
+    print(idx, "user_uid:",item['user_uid'],"\n transaction_uid:",item['transaction_uid'])
     print("... ")
     if 'classification_datetime' in item:
         item['classified_at'] = item['classification_datetime']
@@ -275,65 +331,70 @@ for idx ,item in enumerate(r.to_dict(orient='records')):
         item['language_code'] = 'es-MX'
     else :
         item['language_code'] = 'es-MX'
-        
-
+    
+    
+    response = get_info_by_transaction_uid(item['transaction_uid'])
+    
+    if response is None :
+        continue 
+    
+    print(response)
     # Dict to necessary data
     transaction_dict = Transaction_().get()
     transaction_dict.update(item)
-
-
-    transaction_dict['message_type'] = item['message_type_old']
-    transaction_dict['flow_type'] = ''
     
-    if not transaction_dict['clean_description']:
+    transaction_dict.update(response)
+
+    
+    
+    
+    if not 'message_type' in response:
+        if transaction_dict['message_type_head'] is not None:
+            transaction_dict['message_type'] = transaction_dict['message_type_head'] 
+        elif transaction_dict['message_type_history']  is not None:
+            transaction_dict['message_type'] = transaction_dict['message_type_history'] 
+        elif transaction_dict['message_type_old']  is not None:
+            transaction_dict['message_type'] = transaction_dict['message_type_old']
+        else: 
+            transaction_dict['message_type'] = 'ingest'
+    
+    
+
+
+    if transaction_dict['clean_description'] is None:
         transaction_dict['clean_description'] = ''
 
 
-    if not transaction_dict['source_description']:
-        transaction_dict['source_description'] = ''
 
-    if not transaction_dict['model_classification_version']:
-        transaction_dict['model_classification_version'] = ''
+    if transaction_dict['model_classification_version'] is None:
+        transaction_dict['model_classification_version'] = 'bf24e9bff6ac4750b1daab8cd57f396d'
 
-    if not transaction_dict['model_nickname_version']:
-        transaction_dict['model_nickname_version'] = ''
+    if transaction_dict['model_nickname_version'] is None :
+        transaction_dict['model_nickname_version'] = 'cM8zVveZ8bnlx4fuBLPH0gJZDh1wem_d0GBAv3f7'
 
-    if not transaction_dict['model_nlp_version']:
-        transaction_dict['model_nlp_version'] = ''
+    if transaction_dict['model_nlp_version'] is None:
+        transaction_dict['model_nlp_version'] = '0grVmHxHgmD8-ad2pz9-HiNE1gKN_htomj5ktzSj'
 
-    if not transaction_dict['model_collective_version']:
-        transaction_dict['model_collective_version'] = ''
+    if transaction_dict['model_collective_version'] is None:
+        transaction_dict['model_collective_version'] = '7Sdt1KJFB4DVVV4kqnSO5AEMVJF66q9ASUAAJG'
 
         
     format_inter = '%Y-%m-%dT%H:%M:%S'
     
-    if not transaction_dict['original_created_date'] or transaction_dict['original_created_date'] is pd.NaT:
-        transaction_dict['original_created_date'] = datetime.strptime('2000-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
-
-
+    
     if not transaction_dict['created_at_homologation'] or transaction_dict['created_at_homologation'] is pd.NaT:
-        transaction_dict['created_at_homologation'] = datetime.strptime('2000-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
-
-    if not item['created_at_head'] or item['created_at_head'] is pd.NaT:
-        item['created_at_head'] = datetime.strptime('2000-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
+        transaction_dict['created_at_homologation'] = datetime.strptime('2023-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
         
     if not transaction_dict['classified_at']:
-        transaction_dict['classified_at'] = datetime.strptime('2000-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
+        transaction_dict['classified_at'] = datetime.strptime('2023-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
 
     if not transaction_dict['updated_at']:
-        transaction_dict['updated_at'] = datetime.strptime('2000-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
+        transaction_dict['updated_at'] = datetime.strptime('2023-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
         
         
     if not transaction_dict['created_at']:
-        transaction_dict['created_at'] = datetime.strptime('2000-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
+        transaction_dict['created_at'] = datetime.strptime('2023-01-01T00:00:00',format_inter).replace(microsecond=0).isoformat()
         
-        
-    if not  transaction_dict['message_type'] :
-        transaction_dict['message_type'] = 'NO_FOUND'
-    # print("transaction_dict['created_at']:::::::::::::", transaction_dict['created_at'])
-    # if transaction_dict['created_at'] is pd.NaT:
-    #    print("siiiiiii")
-   
 
     keys = Transaction_().get().keys()
 
@@ -352,25 +413,23 @@ for idx ,item in enumerate(r.to_dict(orient='records')):
     
     user_dict = None
     # new user ?
+    
+    if not RedShift.exists_user(item['user_uid']).values[0][0]:
     #if not MongoDB.exist('user_uid',item['user_uid']):
-    user_dict ={"user_uid":item['user_uid'],
-                "postcode":None
-                }
-    
-    if not 'account_uid' in item:
-        item['account_uid'] = '123'
-    
-    if item['account_uid'] is None:
-        item['account_uid'] = '123'
-
-    account_dict ={
-                    'account_uid': item['account_uid'],
-                    'bank_name':  item['bank_name'],
-                    'account_type': item['account_type'],
-                    'bank_type': item['bank_type']
+        user_dict ={"user_uid":item['user_uid'],
+                    "postcode":None
                     }
-    RedShift.insert_dim_user(dict_values = user_dict)
-    RedShift.insert_dim_account(dict_values = account_dict)
+        RedShift.insert_dim_user(dict_values = user_dict)
+
+
+        account_dict ={
+                    'account_uid': response['account_uid'],
+                    'bank_name':  response['bank_name'],
+                    'account_type': response['account_type'],
+                    'bank_type': response['bank_type']
+                    }
+    
+        RedShift.insert_dim_account(dict_values = account_dict)
     
     print("****************************************\n")
     print(account_dict)
@@ -393,12 +452,13 @@ for idx ,item in enumerate(r.to_dict(orient='records')):
     fact_transactions_dict['transaction_date'] = str(date_dict['transaction_date'])
     fact_transactions_dict['transaction_uid'] = item['transaction_uid']
     fact_transactions_dict['user_uid'] = item['user_uid']
-    fact_transactions_dict['account_uid'] = item['account_uid']
-    fact_transactions_dict['amount'] = item['amount']
+    fact_transactions_dict['account_uid'] = response['account_uid']
+    fact_transactions_dict['amount'] = response['amount']
+    fact_transactions_dict['currency'] = response['currency']
 
 
  
-    print("\nfact_transactions_dict: ")
+    print("\n\nfact_transactions_dict: ")
     print(fact_transactions_dict)
     print("----------------------------------------------")
 
@@ -407,3 +467,7 @@ for idx ,item in enumerate(r.to_dict(orient='records')):
     RedShift.insert_dim_transaction(dict_values = transaction_dict)
 
     RedShift.insert_fact_transactions(dict_values = fact_transactions_dict)
+    
+    print("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
+    print("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
+    
\ No newline at end of file
